package envoy.server.processors;

import java.io.IOException;
import java.util.Arrays;

import envoy.event.contact.ContactOperationEvent;
import envoy.server.data.PersistenceManager;
import envoy.server.net.ConnectionManager;
import envoy.server.net.ObjectWriteProxy;

/**
 * Project: <strong>envoy-server-standalone</strong><br>
 * File: <strong>ContactOperationProcessor.java</strong><br>
 * Created: <strong>08.02.2020</strong><br>
 *
 * @author Kai S. K. Engelbart
 * @since Envoy Server Standalone v0.1-alpha
 */
public class ContactOperationProcessor implements ObjectProcessor<ContactOperationEvent> {

	private static final ConnectionManager connectionManager = ConnectionManager.getInstance();

	@Override
	public void process(ContactOperationEvent evt, long socketId, ObjectWriteProxy writeProxy) throws IOException {
		switch (evt.getOperationType()) {
			case ADD:
				final long userID = ConnectionManager.getInstance().getUserIdBySocketId(socketId);
				final long contactId = evt.get().getID();

				System.out.printf("Adding user %s to the contact list of user %d.%n", evt.get(), userID);
				PersistenceManager.getInstance().addUserContact(userID, contactId);

				// Notify the contact if online
				if (ConnectionManager.getInstance().isOnline(contactId)) writeProxy.write(connectionManager.getSocketId(contactId),
						Arrays.asList(PersistenceManager.getInstance().getUserByID(userID).toCommon()));
				break;
			default:
				System.err.printf("Received %s with an unsupported operation.%n", evt);
		}
	}

	@Override
	public Class<ContactOperationEvent> getInputClass() { return ContactOperationEvent.class; }
}
