package envoy.server.processors;

import java.io.IOException;

import envoy.event.GroupCreationEvent;
import envoy.server.data.Contact;
import envoy.server.data.PersistenceManager;
import envoy.server.net.ConnectionManager;
import envoy.server.net.ObjectWriteProxy;

/**
 * Project: <strong>envoy-server-standalone</strong><br>
 * File: <strong>GroupCreationProcessor.java</strong><br>
 * Created: <strong>26.03.2020</strong><br>
 * 
 * @author Maximilian K&auml;fer
 * @since Envoy Server Standalone v0.1-beta
 */
public class GroupCreationProcessor implements ObjectProcessor<GroupCreationEvent> {

	private final PersistenceManager	persistenceManager	= PersistenceManager.getInstance();
	private final ConnectionManager		connectionManager	= ConnectionManager.getInstance();

	@Override
	public void process(GroupCreationEvent input, long socketID, ObjectWriteProxy writeProxy) throws IOException {
		envoy.server.data.Group group = new envoy.server.data.Group();
		group.setName(input.get());
		input.getInitialMemberIDs().stream().map(persistenceManager::getUserByID).forEach(group.getContacts()::add);
		group.getContacts().add(persistenceManager.getUserByID(connectionManager.getUserIdBySocketId(socketID)));
		persistenceManager.addContact(group);
		writeProxy.write(socketID, group);
		group.getContacts()
			.stream()
			.map(Contact::getID)
			.filter(connectionManager::isOnline)
			.map(connectionManager::getSocketId)
			.forEach(memberSocketID -> {
				try {
					writeProxy.write(memberSocketID, group);
				} catch (IOException e) {
					e.printStackTrace();
				}
			});
	}

	@Override
	public Class<GroupCreationEvent> getInputClass() { return GroupCreationEvent.class; }
}
